=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-10-13 21:29:09 +0000
+++ bzrlib/changeset.py	2005-10-13 21:58:18 +0000
@@ -1064,6 +1064,11 @@
         msg = "Conflicting contents for new file %s" % (filename)
         Exception.__init__(self, msg)
 
+class ThreewayContentsConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
 
 class MissingForMerge(Exception):
     def __init__(self, filename):
@@ -1135,6 +1140,10 @@
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
+    def threeway_contents_conflict(self, filename, this_contents,
+                                   base_contents, other_contents):
+        raise ThreewayContentsConflict(filename)
+
     def finalize(self):
         pass
 

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-10-12 07:28:41 +0000
+++ bzrlib/delta.py	2005-10-13 21:58:18 +0000
@@ -187,8 +187,8 @@
             # get them
             old_path = old_inv.id2path(file_id)
             new_path = new_inv.id2path(file_id)
-            old_ie._read_tree_state(old_tree)
-            new_ie._read_tree_state(new_tree)
+            old_ie._read_tree_state(old_path, old_tree)
+            new_ie._read_tree_state(new_path, new_tree)
             text_modified, meta_modified = new_ie.detect_changes(old_ie)
 
             # TODO: Can possibly avoid calculating path strings if the

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-12 07:28:41 +0000
+++ bzrlib/inventory.py	2005-10-13 21:58:18 +0000
@@ -133,12 +133,13 @@
         text_diff will be used for textual difference calculation.
         This is a template method, override _diff in child classes.
         """
-        self._read_tree_state(tree)
+        self._read_tree_state(tree.id2path(self.file_id), tree)
         if to_entry:
             # cannot diff from one kind to another - you must do a removal
             # and an addif they do not match.
             assert self.kind == to_entry.kind
-            to_entry._read_tree_state(to_tree)
+            to_entry._read_tree_state(to_tree.id2path(to_entry.file_id),
+                                      to_tree)
         self._diff(text_diff, from_label, tree, to_label, to_entry, to_tree,
                    output_to, reverse)
 
@@ -325,7 +326,7 @@
         text stored in the text store or weave.
         """
         mutter('new parents of %s are %r', path, previous_entries)
-        self._read_tree_state(work_tree)
+        self._read_tree_state(path, work_tree)
         if len(previous_entries) == 1:
             # cannot be unchanged unless there is only one parent file rev.
             parent_ie = previous_entries.values()[0]
@@ -391,7 +392,7 @@
             compatible = False
         return compatible
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """Populate fields in the inventory entry from the given tree.
         
         Note that this should be modified to be a noop on virtual trees
@@ -543,7 +544,7 @@
         if tree.is_executable(self.file_id):
             os.chmod(fullpath, 0755)
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.text_sha1 = work_tree.get_file_sha1(self.file_id)
         self.executable = work_tree.is_executable(self.file_id)
@@ -653,7 +654,7 @@
         except OSError,e:
             raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
 
-    def _read_tree_state(self, work_tree):
+    def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.symlink_target = work_tree.get_symlink_target(self.file_id)
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-13 21:29:09 +0000
+++ bzrlib/merge.py	2005-10-13 21:58:18 +0000
@@ -190,6 +190,10 @@
         self.create(file_id, stem+".OTHER", self.other_tree)
         self.create(file_id, stem+".BASE", self.base_tree)
 
+    def threeway_contents_conflict(filename, this_contents, base_contents,
+                                   other_contents):
+        self.conflict("Three-way conflict merging %s" % filename)
+
     def finalize(self):
         if not self.ignore_zero:
             note("%d conflicts encountered.\n" % self.conflicts)

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-10-13 21:29:09 +0000
+++ bzrlib/merge_core.py	2005-10-13 21:58:18 +0000
@@ -42,11 +42,6 @@
                 raise Exception("%s not in tree" % self.file_id)
                 return ()
             return tree.get_file(self.file_id).readlines()
-        ### garh. 
-        other_entry = other.tree.inventory[self.file_id]
-        if other_entry.kind == 'symlink':
-            self.apply_symlink(other_entry, base, other, filename)
-            return
         base_lines = get_lines(base)
         other_lines = get_lines(other)
         m3 = Merge3(base_lines, file(filename, "rb").readlines(), other_lines)
@@ -70,36 +65,6 @@
             conflict_handler.merge_conflict(new_file, filename, base_lines,
                                             other_lines)
 
-    def apply_symlink(self, other_entry, base, other, filename):
-        if self.file_id in base:
-            base_entry = base.tree.inventory[self.file_id]
-            base_entry._read_tree_state(base.tree)
-        else:
-            base_entry = None
-        other_entry._read_tree_state(other.tree)
-        if not base_entry or other_entry.detect_changes(base_entry):
-            other_change = True
-        else:
-            other_change = False
-        this_link = os.readlink(filename)
-        if not base_entry or base_entry.symlink_target != this_link:
-            this_change = True
-        else:
-            this_change = False
-        if this_change and not other_change:
-            pass
-        elif not this_change and other_change:
-            os.unlink(filename)
-            os.symlink(other_entry.symlink_target, filename)
-        elif this_change and other_change:
-            # conflict
-            os.unlink(filename)
-            os.symlink(other_entry.symlink_target, filename + '.OTHER')
-            os.symlink(this_link, filename + '.THIS')
-            if base_entry is not None:
-                os.symlink(other_entry.symlink_target, filename + '.BASE')
-            note("merge3 conflict in '%s'.\n", filename)
-
 
 class BackupBeforeChange:
     """Contents-change wrapper to back up file first"""
@@ -278,11 +243,18 @@
         elif isinstance(contents.old_contents, changeset.TreeFileCreate) and \
             isinstance(contents.new_contents, changeset.TreeFileCreate):
             return make_merge()
-        elif (isinstance(contents.old_contents, changeset.SymlinkCreate)
-              and isinstance(contents.new_contents, changeset.SymlinkCreate)):
-            return make_merge()
         else:
-            raise Exception("Unhandled merge scenario %r" % contents)
+            this_contents = get_contents(this, entry.id)
+            if this_contents == contents.old_contents:
+                return contents
+            elif this_contents == contents.new_contents:
+                return None
+            elif contents.old_contents == contents.new_contents:
+                return None
+            else:
+                conflict_handler.threeway_contents_conflict(this_contents,
+                    contents.old_contents, contents.new_contents)
+                
 
 def make_merged_metadata(entry, base, other):
     metadata = entry.metadata_change

